@@ -3,7 +3,7 @@ import platform
 import subprocess
 from pathlib import Path
 from typing import Any, Dict, List, Optional, Tuple, Type, Union
-
+import os 
 from ninja.ninja_syntax import Writer
 
 from ccimport import compat
@@ -13,6 +13,27 @@ ALL_SUPPORTED_LINKER = set(['cl', 'nvcc', 'g++', 'clang++'])
 
 _ALL_OVERRIDE_FLAGS = (set(["/MT", "/MD", "/LD", "/MTd", "/MDd", "/LDd"]), )
 
+def _make_unique_name(unique_set, name, max_count=10000):
+    if name not in unique_set:
+        unique_set.add(name)
+        return name
+    for i in range(max_count):
+        new_name = name + "_{}".format(i)
+        if new_name not in unique_set:
+            unique_set.add(new_name)
+            return new_name
+    raise ValueError("max count reached")
+
+
+class UniqueNamePool:
+    def __init__(self, max_count=10000):
+        self.max_count = max_count
+        self.unique_set = set()
+
+    def __call__(self, name):
+        return _make_unique_name(self.unique_set, name, self.max_count)
+
+
 
 def _list_none(val):
     if val is None:
@@ -35,6 +56,23 @@ def _filter_unsupported_compiler(compilers: List[str]):
             supported.append(c)
     return supported
 
+def _override_flags(major_flags, minor_flags):
+    """if a flag exists in _ALL_OVERRIDE_FLAGS, the one
+    in major flag will override the one in minor.
+    """
+    flag_to_override = set([])
+    for override_flags in _ALL_OVERRIDE_FLAGS:
+        for flag in major_flags:
+            if flag in override_flags:
+                for flag2 in minor_flags:
+                    if flag2 in override_flags:
+                        flag_to_override.add(flag2)
+    new_flags = []
+    for flag2 in minor_flags:
+        if flag2 not in flag_to_override:
+            new_flags.append(flag2)
+    return new_flags
+
 
 class BuildOptions:
     def __init__(self,
@@ -49,26 +87,13 @@ class BuildOptions:
         return BuildOptions(self.includes.copy(), self.cflags.copy(),
                             self.post_cflags.copy())
 
-    def _override_flags(self, major_flags, minor_flags):
-        flag_to_override = set([])
-        for override_flags in _ALL_OVERRIDE_FLAGS:
-            for flag in major_flags:
-                if flag in override_flags:
-                    for flag2 in minor_flags:
-                        if flag2 in override_flags:
-                            flag_to_override.add(flag2)
-        new_flags = []
-        for flag2 in minor_flags:
-            if flag2 not in flag_to_override:
-                new_flags.append(flag2)
-        return new_flags
 
     def merge(self, opt: "BuildOptions"):
         res = self.copy()
         res.includes.extend(opt.includes)
-        res.cflags.extend(self._override_flags(res.cflags, opt.cflags))
+        res.cflags.extend(_override_flags(res.cflags, opt.cflags))
         res.post_cflags.extend(
-            self._override_flags(res.post_cflags, opt.post_cflags))
+            _override_flags(res.post_cflags, opt.post_cflags))
         return res
 
 
@@ -126,6 +151,7 @@ class BaseWritter(Writer):
 
             self._suffix_to_cl[suffix] = compiler_name
             self._compiler_var_to_name[compiler_name] = compiler
+        self.variable("msvc_deps_prefix",  os.getenv("CCIMPORT_MSVC_DEPS_PREFIX", "Note: including file:"))
 
     @property
     def content(self) -> str:
@@ -194,7 +220,7 @@ class BaseWritter(Writer):
         rule_name = name + "_cxx_{}".format(compiler_var)
         self.rule(
             rule_name,
-            "${} {} {} /showIncludes -c $in /Fo$out ${}".format(compiler_var, includes, cflags, post_cflags),
+            "${} {} {} /showIncludes -c $in /Fo$out {}".format(compiler_var, includes, cflags, post_cflags),
             deps="msvc"
         )
         self.newline()
@@ -340,10 +366,12 @@ class BaseWritter(Writer):
         else:
             target_path = self._build_dir / target_filename
         obj_files = []
+        name_pool = UniqueNamePool()
         for p in source_paths:
             assert p.exists()
             suffix = ".o"
-            obj = (self._build_dir / (p.name + suffix)).resolve()
+            file_name = name_pool(p.name)
+            obj = (self._build_dir / (file_name + suffix))
             assert obj.parent.exists()
             obj = str(obj)
             obj_files.append(obj)
@@ -388,13 +416,13 @@ COMMON_NVCC_FLAGS = [
 ]
 
 COMMON_NVCC_FLAGS_WINDOWS = [
-    '--expt-relaxed-constexpr', '-Xcompiler=\'/O2\''
+    '--expt-relaxed-constexpr', '-Xcompiler=\"/O2\"'
 ]
 
 
 COMMON_MSVC_FLAGS = [
     '/MD', '/wd4819', '/wd4251', '/wd4244', '/wd4267', '/wd4275', '/wd4018',
-    '/wd4190', '/EHsc'
+    '/wd4190', '/EHsc', '/Zc:__cplusplus'
 ]
 
 COMMON_HIPCC_FLAGS = [
